searchData={"items":[{"type":"module","title":"Parameter","doc":"`Parameter` helps you shape data from external sources into Elixir internal types. Use it to deal with any external data in general, such as API integrations, parsing user input, or validating data that comes into your system.\n\n`Parameter` offers the following helpers:\n- Schema creation and validation\n- Input data validation\n- Deserialization\n- Serialization","ref":"Parameter.html"},{"type":"module","title":"Schema - Parameter","doc":"First step for dealing with external data is to create a schema that shape the data:\n\n    defmodule UserParam do\n      use Parameter.Schema\n      alias Parameter.Validators\n\n      param do\n        field :first_name, :string, key: \"firstName\", required: true\n        field :last_name, :string, key: \"lastName\"\n        field :email, :string, validator: &Validators.email/1\n        has_one :address, AddressParam do\n          field :city, :string, required: true\n          field :street, :string\n          field :number, :integer\n        end\n      end\n    end\n\nNow it's possible to Load (deserialize) the schema against external data:\n\n    params = %{\n      \"firstName\" => \"John\",\n      \"lastName\" => \"Doe\",\n      \"email\" => \"john@email.com\",\n      \"address\" => %{\"city\" => \"New York\", \"street\" => \"York\"}\n    }\n    Parameter.load(UserParam, params)\n    {:ok, %{\n      first_name: \"John\",\n      last_name: \"Doe\",\n      email: \"john@email.com\",\n      address: %{city: \"New York\", street: \"York\"}\n    }}\n\nor Dump (serialize) a populated schema to the source:\n\n    schema = %{\n      first_name: \"John\",\n      last_name: \"Doe\",\n      email: \"john@email.com\",\n      address: %{city: \"New York\", street: \"York\"}\n    }\n    Parameter.dump(UserParam, params)\n    {:ok, %{\n      \"firstName\" => \"John\",\n      \"lastName\" => \"Doe\",\n      \"email\" => \"john@email.com\",\n      \"address\" => %{\"city\" => \"New York\", \"street\" => \"York\"}\n    }}\n\nFor more schema options checkout `Parameter.Schema`","ref":"Parameter.html#module-schema"},{"type":"function","title":"Parameter.dump/3","doc":"Dump the loaded parameters.","ref":"Parameter.html#dump/3"},{"type":"function","title":"Options - Parameter.dump/3","doc":"* `:exclude` - Accepts a list of fields to be excluded when dumping the loaded parameter. This\n  option is useful if you have fields in your schema are only for loading.\n\n  * `:ignore_nil` - When `true` will ignore `nil` values when dumping the parameters, when `false` (default) it will dump the `nil` values.\n\n  * `:ignore_empty` - When `true` will ignore empty `\"\"` values when dumping the parameters, when `false` (default) it will dump the empty values.\n\n  * `:many` - When `true` will parse the input data as list, when `false` (default) it parses as map","ref":"Parameter.html#dump/3-options"},{"type":"function","title":"Examples - Parameter.dump/3","doc":"defmodule UserParam do\n      use Parameter.Schema\n\n      param do\n        field :first_name, :string, key: \"firstName\", required: true\n        field :last_name, :string, key: \"lastName\"\n        has_one :address, Address do\n          field :city, :string, required: true\n          field :street, :string\n          field :number, :integer\n        end\n      end\n    end\n\n    loaded_params = %{\n      first_name: \"John\",\n      last_name: \"Doe\",\n      address: %{city: \"New York\", street: \"broadway\"}\n    }\n\n    Parameter.dump(UserParam, params)\n    {:ok, %{\n      \"address\" => %{\"city\" => \"New York\", \"street\" => \"broadway\"},\n      \"firstName\" => \"John\",\n      \"lastName\" => \"Doe\"\n    }}\n\n    # excluding fields\n    Parameter.dump(UserParam, params, exclude: [:first_name, {:address, [:city]}])\n    {:ok, %{\n      \"address\" => %{\"street\" => \"broadway\"},\n      \"lastName\" => \"Doe\"\n    }}","ref":"Parameter.html#dump/3-examples"},{"type":"function","title":"Parameter.load/3","doc":"Loads parameters into the given schema.","ref":"Parameter.html#load/3"},{"type":"function","title":"Options - Parameter.load/3","doc":"* `:struct` - If set to `true` loads the schema into a structure. If `false` (default)\n  loads with plain maps.\n\n  * `:unknown` - Defines the behaviour when unknown fields are presented on the parameters.\n  The options are `:ignore` (default) or `:error`.\n\n  * `:exclude` - Accepts a list of fields to be excluded when loading the parameters. This\n  option is useful if you have fields in your schema are only for dump. The field will not\n  be checked for any validation if it's on the exclude list.\n\n  * `:ignore_nil` - When `true` will ignore `nil` values when loading the parameters, when `false` (default) it will load the `nil` values.\n\n  * `:ignore_empty` - When `true` will ignore empty `\"\"` values when loading the parameters, when `false` (default) it will load the empty values.\n\n  * `:many` - When `true` will parse the input data as list, when `false` (default) it parses as map","ref":"Parameter.html#load/3-options"},{"type":"function","title":"Examples - Parameter.load/3","doc":"defmodule UserParam do\n      use Parameter.Schema\n\n      param do\n        field :first_name, :string, key: \"firstName\", required: true\n        field :last_name, :string, key: \"lastName\"\n        has_one :address, Address do\n          field :city, :string, required: true\n          field :street, :string\n          field :number, :integer\n        end\n      end\n    end\n\n    params = %{\n      \"address\" => %{\"city\" => \"New York\", \"street\" => \"broadway\"},\n      \"firstName\" => \"John\",\n      \"lastName\" => \"Doe\"\n    }\n    Parameter.load(UserParam, params)\n    {:ok, %{\n      first_name: \"John\",\n      last_name: \"Doe\",\n      address: %{city: \"New York\", street: \"broadway\"}\n    }}\n\n    # Using struct options\n    Parameter.load(UserParam, params, struct: true)\n    {:ok, %UserParam{\n      first_name: \"John\",\n      last_name: \"Doe\",\n      address: %AddressParam{city: \"New York\", street: \"broadway\"}\n    }}\n\n    # Using many: true for lists\n    Parameter.load(UserParam, [params, params], many: true)\n    {:ok,\n    [\n      %{\n        address: %{city: \"New York\", street: \"broadway\"},\n        first_name: \"John\",\n        last_name: \"Doe\"\n      },\n      %{\n        address: %{city: \"New York\", street: \"broadway\"},\n        first_name: \"John\",\n        last_name: \"Doe\"\n      }\n    ]}\n\n    # Excluding fields\n    Parameter.load(UserParam, params, exclude: [:first_name, {:address, [:city]}])\n    {:ok, %{\n      last_name: \"Doe\",\n      address: %{street: \"broadway\"}\n    }}\n\n    # Unknown fields should return errors\n    params = %{\"user_token\" => \"3hgj81312312\"}\n    Parameter.load(UserParam, params, unknown: :error)\n    {:error, %{\"user_token\" => \"unknown field\"}}\n\n    # Invalid data should return validation errors:\n    params = %{\n      \"address\" => %{\"city\" => \"New York\", \"number\" => \"123AB\"},\n      \"lastName\" => \"Doe\"\n    }\n    Parameter.load(UserParam, params)\n    {:error, %{\n      first_name: \"is required\",\n      address: %{number: \"invalid integer type\"},\n    }}\n\n    Parameter.load(UserParam, [params, params], many: true)\n    {:error,\n    %{\n      0 => %{address: %{number: \"invalid integer type\"}, first_name: \"is required\"},\n      1 => %{address: %{number: \"invalid integer type\"}, first_name: \"is required\"}\n    }}\n\n#","ref":"Parameter.html#load/3-examples"},{"type":"function","title":"Loading map with atom keys - Parameter.load/3","doc":"It's also possible to load map with atom keys. Parameter schemas should not implement the `key`\noption for this to work.\n\n    defmodule UserParam do\n      use Parameter.Schema\n\n      param do\n        field :first_name, :string, required: true\n        field :last_name, :string\n      end\n    end\n\n    params = %{\n      first_name: \"John\",\n      last_name: \"Doe\"\n    }\n    Parameter.load(UserParam, params)\n    {:ok, %{\n      first_name: \"John\",\n      last_name: \"Doe\"\n    }}\n\n    # String maps will also be correctly loaded if they have the same key\n    params = %{\n      \"first_name\" => \"John\",\n      \"last_name\" => \"Doe\"\n    }\n    Parameter.load(UserParam, params)\n    {:ok, %{\n      first_name: \"John\",\n      last_name: \"Doe\"\n    }}\n\n    # But the same key should not be present in both String and Atom keys:\n    params = %{\n      \"first_name\" => \"John\",\n      first_name: \"John\"\n    }\n    Parameter.load(UserParam, params)\n    {:error, %{\n      first_name: \"field is present as atom and string keys\"\n    }}","ref":"Parameter.html#load/3-loading-map-with-atom-keys"},{"type":"function","title":"Parameter.validate/3","doc":"Validate parameters. This function is meant to be used when the data is loaded or\ncreated internally. `validate/3` will validate field types, required fields and\n`Parameter.Validators` functions.","ref":"Parameter.html#validate/3"},{"type":"function","title":"Options - Parameter.validate/3","doc":"* `:exclude` - Accepts a list of fields to be excluded when validating the parameters.\n\n  * `:many` - When `true` will parse the input data as list, when `false` (default) it parses as map","ref":"Parameter.html#validate/3-options"},{"type":"function","title":"Examples - Parameter.validate/3","doc":"defmodule UserParam do\n      use Parameter.Schema\n\n      param do\n        field :first_name, :string, key: \"firstName\", required: true\n        field :last_name, :string, key: \"lastName\"\n        has_one :address, Address do\n          field :city, :string, required: true\n          field :street, :string\n          field :number, :integer\n        end\n      end\n    end\n\n    params = %{\n      first_name: \"John\",\n      last_name: \"Doe\",\n      address: %{city: \"New York\", street: \"broadway\"}\n    }\n\n    Parameter.validate(UserParam, params)\n    :ok\n\n    # Invalid data\n    params = %{\n      last_name: 12,\n      address: %{city: \"New York\", street: \"broadway\", number: \"A\"}\n    }\n\n    Parameter.validate(UserParam, params)\n    {:error,\n      %{\n        address: %{number: \"invalid integer type\"},\n        first_name: \"is required\",\n        last_name: \"invalid string type\"\n      }\n    }","ref":"Parameter.html#validate/3-examples"},{"type":"module","title":"Parameter.Field","doc":"The field inside a Parameter Schema have the following structure:\n    field :name, :type, opts\n\n* `:name` - Atom key that defines the field name\n* `:type` - Type from `Parameter.Types`. For custom types check the `Parameter.Parametrizable` behaviour.\n* `:opts` - Keyword with field options.","ref":"Parameter.Field.html"},{"type":"module","title":"Options - Parameter.Field","doc":"* `:key` - This is the key from the params that will be converted to the field schema. Examples:\n  * If an input field use `camelCase` for mapping `first_name`, this option should be set as \"firstName\".\n  * If an input field use the same case for the field definition, this key can be ignored.\n\n* `:default` - Default value of the field when no value is given.\n\n* `:load_default` - Default value of the field when no value is given when loading with `Parameter.load/3` function.\nThis option should not be used at the same time as `default` option.\n\n* `:dump_default` - Default value of the field when no value is given when loading with `Parameter.dump/3` function.\nThis option should not be used at the same time as `default` option.\n\n* `:required` - Defines if the field needs to be present when parsing the input.\n`Parameter.load/3` will return an error if the value is missing from the input data.\n\n* `:validator` - Validation function that will validate the field after loading.\n\n* `:virtual` - If `true` the field will be ignored on `Parameter.load/3` and `Parameter.dump/3` functions.\n\n* `:on_load` - Function to specify how to load the field. The function must have two arguments where the first one is the field value and the second one\nwill be the data to be loaded. Should return `{:ok, value}` or `{:error, reason}` tuple.\n\n* `:on_dump` - Function to specify how to dump the field. The function must have two arguments where the first one is the field value and the second one\nwill be the data to be dumped. Should return `{:ok, value}` or `{:error, reason}` tuple.\n\n> NOTE: Validation only occurs on `Parameter.load/3`.\n> By desgin, data passed into `Parameter.dump/3` are considered valid.","ref":"Parameter.Field.html#module-options"},{"type":"module","title":"Example - Parameter.Field","doc":"As an example having an `email` field that is required and needs email validation could be implemented this way:\n    field :email, :string, required: true, validator: &Parameter.Validators.email/1","ref":"Parameter.Field.html#module-example"},{"type":"type","title":"Parameter.Field.t/0","doc":"","ref":"Parameter.Field.html#t:t/0"},{"type":"module","title":"Parameter.Schema","doc":"The first step for building a schema for your data is to create a schema definition to model the external data.\nThis can be achieved by using the `Parameter.Schema` macro.","ref":"Parameter.Schema.html"},{"type":"module","title":"Schema - Parameter.Schema","doc":"The example below mimics an `User` model that have one `main_address` and a list of `phones`.\n\n    defmodule User do\n      use Parameter.Schema\n\n      param do\n        field :first_name, :string, key: \"firstName\", required: true\n        field :last_name, :string, key: \"lastName\", required: true, default: \"\"\n        has_one :main_address, Address, key: \"mainAddress\", required: true\n        has_many :phones, Phone\n      end\n    end\n\n    defmodule Address do\n      use Parameter.Schema\n\n      param do\n        field :city, :string, required: true\n        field :street, :string\n        field :number, :integer\n      end\n    end\n\n    defmodule Phone do\n      use Parameter.Schema\n\n      param do\n        field :country, :string\n        field :number, :integer\n      end\n    end\n\n`Parameter` offers other ways for creating a schema such as nesting the `has_one` and `has_many` fields. This require module name as the second parameter using `do` at the end:\n\n    defmodule User do\n      use Parameter.Schema\n\n      param do\n        field :first_name, :string, key: \"firstName\", required: true\n        field :last_name, :string, key: \"lastName\", required: true, default: \"\"\n\n        has_one :main_address, Address, key: \"mainAddress\", required: true do\n          field :city, :string, required: true\n          field :street, :string\n          field :number, :integer\n        end\n\n        has_many :phones, Phone do\n          field :country, :string\n          field :number, :integer\n        end\n      end\n    end\n\nAnother possibility is avoiding creating files for a schema at all. This can be done by importing `Parameter.Schema` and using the `param/2` macro. This is useful for adding params in Phoenix controllers. For example:\n\n    defmodule MyProjectWeb.UserController do\n      use MyProjectWeb, :controller\n      import Parameter.Schema\n\n      alias MyProject.Users\n\n      param UserParams do\n        field :first_name, :string, required: true\n        field :last_name, :string, required: true\n      end\n\n      def create(conn, params) do\n        with {:ok, user_params} <- Parameter.load(__MODULE__.UserParams, params),\n            {:ok, user} <- Users.create_user(user_params) do\n          render(conn, \"user.json\", %{user: user})\n        end\n      end\n    end\n\nIt's recommended to use this approach when the schema will only be used in a single module.\n\n> #### `use Parameter.Schema` {: .info}\n>\n> When you `use Parameter.Schema`, the `Schema` module will register\n> module attributes and inject functions that are necessary for\n> `Parameter.load/3`, `Parameter.validate/3` and `Parameter.dump/3` to fetch the schema\n> on runtime.","ref":"Parameter.Schema.html#module-schema"},{"type":"module","title":"Runtime Schemas - Parameter.Schema","doc":"It's also possible to create schemas via runtime without relying on any macros.\nThe API is almost the same comparing to the macro's examples:\n\n    schema = %{\n      first_name: [key: \"firstName\", type: :string, required: true],\n      address: [type: {:map, %{street: [type: :string, required: true]}}],\n      phones: [type: {:array, %{country: [type: :string, required: true]}}]\n    } |> Parameter.Schema.compile!()\n\n    Parameter.load(schema, %{\"firstName\" => \"John\"})\n    {:ok, %{first_name: \"John\"}}\n\n\n  The same API can also be evaluated on compile time by using module attributes:\n\n    defmodule UserParams do\n      alias Parameter.Schema\n\n      @schema %{\n        first_name: [key: \"firstName\", type: :string, required: true],\n        address: [required: true, type: {:map, %{street: [type: :string, required: true]}}],\n        phones: [type: {:array, %{country: [type: :string, required: true]}}]\n      } |> Schema.compile!()\n\n      def load(params) do\n        Parameter.load(@schema, params)\n      end\n    end\n\n  This makes it easy to dynamically create schemas or just avoid using any macros.","ref":"Parameter.Schema.html#module-runtime-schemas"},{"type":"module","title":"Required fields - Parameter.Schema","doc":"By default, `Parameter.Schema` considers all fields to be optional when validating the schema.\nThis behaviour can be changed by passing the module attribute `@fields_required true` on\nthe module where the schema is declared.\n\n#","ref":"Parameter.Schema.html#module-required-fields"},{"type":"module","title":"Example - Parameter.Schema","doc":"defmodule MyApp.UserSchema do\n      use Parameter.Schema\n\n      @fields_required true\n\n      param do\n        field :name, :string\n        field :age, :integer\n      end\n    end\n\n    Parameter.load(MyApp.UserSchema, %{})\n    {:error, %{age: \"is required\", name: \"is required\"}}","ref":"Parameter.Schema.html#module-example"},{"type":"module","title":"Custom field loading and dumping - Parameter.Schema","doc":"The `load` and `dump` behavior can be customized per field by implementing `on_load` or `on_dump` functions in the field definition.\nThis can be useful if the field needs to be fetched or even validate in a different way than the defaults implemented by `Parameter`.\nBoth functions should return `{:ok, value}` or `{:error, reason}` tuple.\n\nFor example, imagine that there is a parameter called `full_name` in your schema that you want to customize on how it will be parsed:\n\n    defmodule MyApp.UserSchema do\n      use Parameter.Schema\n\n      param do\n        field :first_name, :string\n        field :last_name, :string\n        field :full_name, :string, on_load: &__MODULE__.load_full_name/2\n      end\n\n      def load_full_name(value, params) do\n        # if `full_name` is not `nil` it just return the `full_name`\n        if value do\n          {:ok, value}\n        else\n          # Otherwise it will join the `first_name` and `last_name` params\n          {:ok, params[\"first_name\"] <> \" \" <> params[\"last_name\"]}\n        end\n      end\n    end\n\nNow when loading, the full_name field will be handled by the `load_full_name/2` function:\n\n    Parameter.load(MyApp.UserSchema, %{first_name: \"John\", last_name: \"Doe\", full_name: nil})\n    {:ok, %{first_name:  \"John\", full_name: \"John Doe\", last_name: \"Doe\"}}\n\nThe same behavior is possible when dumping the schema parameters by using `on_dump/2` function:\n\n    schema = %{\n      level: [type: :integer, on_dump: fn value, _input -> {:ok, value || 0}  end]\n    } |> Parameter.Schema.compile!()\n\n    Parameter.dump(schema, %{level: nil})\n    {:ok, %{\"level\" => 0}}","ref":"Parameter.Schema.html#module-custom-field-loading-and-dumping"},{"type":"function","title":"Parameter.Schema.compile!/1","doc":"","ref":"Parameter.Schema.html#compile!/1"},{"type":"function","title":"Parameter.Schema.field_key/2","doc":"","ref":"Parameter.Schema.html#field_key/2"},{"type":"function","title":"Parameter.Schema.field_keys/1","doc":"","ref":"Parameter.Schema.html#field_keys/1"},{"type":"function","title":"Parameter.Schema.fields/1","doc":"","ref":"Parameter.Schema.html#fields/1"},{"type":"module","title":"Parameter.Types","doc":"Parameter supports different types to be used in the field inside a schema. The available types are:\n\n* `string`\n* `atom`\n* `any`\n* `integer`\n* `float`\n* `boolean`\n* `map`\n* `{map, nested_type}`\n* `array`\n* `{array, nested_type}`\n* `date`\n* `time`\n* `datetime`\n* `naive_datetime`\n* `decimal`*\n* `enum`**\n\n\n\\* For decimal type add the [decimal](https://hexdocs.pm/decimal) library into your project.\n\n\\*\\* Check the `Parameter.Enum` for more information on how to use enums.\n\nFor implementing custom types check the `Parameter.Parametrizable` module. Implementing this behavour in a module makes eligible to be a field in the schema definition.","ref":"Parameter.Types.html"},{"type":"function","title":"Parameter.Types.base_type?/1","doc":"","ref":"Parameter.Types.html#base_type?/1"},{"type":"function","title":"Parameter.Types.composite_inner_type?/1","doc":"","ref":"Parameter.Types.html#composite_inner_type?/1"},{"type":"function","title":"Parameter.Types.composite_type?/1","doc":"","ref":"Parameter.Types.html#composite_type?/1"},{"type":"function","title":"Parameter.Types.dump/2","doc":"","ref":"Parameter.Types.html#dump/2"},{"type":"function","title":"Parameter.Types.load/2","doc":"","ref":"Parameter.Types.html#load/2"},{"type":"function","title":"Parameter.Types.validate/2","doc":"","ref":"Parameter.Types.html#validate/2"},{"type":"function","title":"Parameter.Types.validate!/2","doc":"","ref":"Parameter.Types.html#validate!/2"},{"type":"type","title":"Parameter.Types.base_types/0","doc":"","ref":"Parameter.Types.html#t:base_types/0"},{"type":"type","title":"Parameter.Types.composite_types/0","doc":"","ref":"Parameter.Types.html#t:composite_types/0"},{"type":"type","title":"Parameter.Types.t/0","doc":"","ref":"Parameter.Types.html#t:t/0"},{"type":"module","title":"Parameter.Validators","doc":"Common validators to use within fields of the `Parameter.Schema`","ref":"Parameter.Validators.html"},{"type":"module","title":"Examples - Parameter.Validators","doc":"param User do\n      field :email, :string, validator: &Validators.email(&1)\n    end\n\n    iex> Parameter.load(User, %{\"email\" => \"not an email\"})\n    {:error, %{email: \"is invalid\"}}","ref":"Parameter.Validators.html#module-examples"},{"type":"function","title":"Parameter.Validators.email/1","doc":"Validates email address\n    param User do\n      field :email, :string, validator: &Validators.email(&1)\n    end\n\n    iex> Parameter.load(User, %{\"email\" => \"not an email\"})\n    {:error, %{email: \"is invalid\"}}\n\n    iex> Parameter.load(User, %{\"email\" => \"john@gmail.com\"})\n    {:ok, %{email: \"john@gmail.com\"}}","ref":"Parameter.Validators.html#email/1"},{"type":"function","title":"Parameter.Validators.equal/2","doc":"Validates if value is equal to another\n    param User do\n      field :permission, :string, validator: {&Validators.equal/2, to: \"admin\"}\n    end\n\n    iex> Parameter.load(User, %{\"permission\" => \"super_admin\"})\n    {:error, %{permission: \"is invalid\"}}\n\n    iex> Parameter.load(User, %{\"permission\" => \"admin\"})\n    {:ok, %{permission: \"admin\"}}","ref":"Parameter.Validators.html#equal/2"},{"type":"function","title":"Parameter.Validators.length/2","doc":"Validates if a value is between a min and max\n    param User do\n      field :age, :integer, validator: {&Validators.length/2, min: 18, max: 50}\n    end\n\n    iex> Parameter.load(User, %{\"age\" => 12})\n    {:error, %{age: \"is invalid\"}}\n\n    iex> Parameter.load(User, %{\"age\" => 30})\n    {:ok, %{age: 30}}","ref":"Parameter.Validators.html#length/2"},{"type":"function","title":"Parameter.Validators.none_of/2","doc":"Validates if a value is not a member\n    param User do\n      field :permission, :atom, validator: {&Validators.none_of/2, options: [:admin, :super_admin]}\n    end\n\n    iex> Parameter.load(User, %{\"permission\" => \"super_admin\"})\n    {:error, %{permission: \"is invalid\"}}\n\n    iex> Parameter.load(User, %{\"permission\" => \"normal_user\"})\n    {:ok, %{permission: :normal_user}}","ref":"Parameter.Validators.html#none_of/2"},{"type":"function","title":"Parameter.Validators.one_of/2","doc":"Validates if a value is a member of the list\n    param User do\n      field :permission, :atom, validator: {&Validators.one_of/2, options: [:admin, :super_admin]}\n    end\n\n    iex> Parameter.load(User, %{\"permission\" => \"normal_user\"})\n    {:error, %{permission: \"is invalid\"}}\n\n    iex> Parameter.load(User, %{\"permission\" => \"super_admin\"})\n    {:ok, %{permission: :super_admin}}","ref":"Parameter.Validators.html#one_of/2"},{"type":"function","title":"Parameter.Validators.regex/2","doc":"Validates if a value matches a regex expression\n    param User do\n      field :code, :string, validator: {&Validators.regex/2, regex: ~r/code/}\n    end\n\n    iex> Parameter.load(User, %{\"code\" => \"12345\"})\n    {:error, %{code: \"is invalid\"}}\n\n    iex> Parameter.load(User, %{\"code\" => \"code:12345\"})\n    {:ok, %{code: \"code:12345\"}}","ref":"Parameter.Validators.html#regex/2"},{"type":"type","title":"Parameter.Validators.resp/0","doc":"","ref":"Parameter.Validators.html#t:resp/0"},{"type":"module","title":"Parameter.Enum","doc":"Enum type represents a group of constants that have a value with an associated key.","ref":"Parameter.Enum.html"},{"type":"module","title":"Examples - Parameter.Enum","doc":"defmodule MyApp.UserParam do\n      use Parameter.Schema\n\n      enum Status do\n        value :user_online, key: \"userOnline\"\n        value :user_offline, key: \"userOffline\"\n      end\n\n      param do\n        field :first_name, :string, key: \"firstName\"\n        field :status, MyApp.UserParam.Status\n      end\n    end\n\n  The `Status` enum should automatically translate the `userOnline` and `userOffline` values when loading\n  to the respective atom values.\n    Parameter.load(MyApp.UserParam, %{\"firstName\" => \"John\", \"status\" => \"userOnline\"})\n    {:ok, %{first_name: \"John\", status: :user_online}}\n\n    Parameter.dump(MyApp.UserParam, %{first_name: \"John\", status: :user_online})\n    {:ok, %{\"firstName\" => \"John\", \"status\" => \"userOnline\"}}\n\n\n> #### `Using enum` {: .info}\n>\n> When you use the `enum` macro, `Parameter` creates a module under the hood, injecting under the current module.\n> For this reason, when referencing the enum in a Parameter field, it's required to use the full module name as shown in the examples.\n\n  Enum also supports a shorter version if the key and value are already the same:\n\n    defmodule MyApp.UserParam do\n      ...\n      enum Status, values: [:user_online,  :user_offline]\n      ...\n    end\n\n    Parameter.load(MyApp.UserParam, %{\"firstName\" => \"John\", \"status\" => \"user_online\"})\n    {:ok, %{first_name: \"John\", status: :user_online}}\n\n  Using numbers is also allowed in enums:\n\n    enum Status do\n      value :active, key: 1\n      value :pending_request, key: 2\n    end\n\n    Parameter.load(MyApp.UserParam, %{\"status\" => 1})\n    {:ok, %{status: :active}}\n\n  It's also possible to create enums in different modules by using the\n  `enum/1` macro:\n\n    defmodule MyApp.Status do\n      import Parameter.Enum\n\n      enum do\n        value :user_online, key: \"userOnline\"\n        value :user_offline, key: \"userOffline\"\n      end\n    end\n\n    defmodule MyApp.UserParam do\n      use Parameter.Schema\n      alias MyApp.Status\n\n      param do\n        field :first_name, :string, key: \"firstName\"\n        field :status, Status\n      end\n    end\n\n  And the short version:\n\n    enum values: [:user_online,  :user_offline]","ref":"Parameter.Enum.html#module-examples"},{"type":"module","title":"Dump and validate - Parameter.Enum","doc":"Enums can also be used for validate and dump the data. The `Parameter.validate/3` function will do strict validation, checking if the value correspond to the enum values, which are internally stored as atoms.\n`Parameter.dump/3` will stringify the enum atom value. By design the `Parameter.dump/3` doesn't perform strict validations but for enums, it checks at least if the value exists in the enum definition before dumping.\n\nConsider the following `Parameter.Enum` implementation:\n\n    defmodule Currency do\n      use Parameter.Schema\n      enum Currencies, values: [:EUR, :USD]\n      \n      param do\n        field :currency, __MODULE__.Currencies\n      end\n    end\n\nIt's possible to check if the value provided it's a valid enum in parameter:\n\n    iex> Parameter.validate(Currency, %{currency: :EUR})\n    :ok\n    iex> Parameter.validate(Currency, %{currency: :BRL})\n    {:error, %{currency: \"invalid enum type\"}}\n    # Using the string version should also return an error since it's expected enum values to be atoms\n    iex> Parameter.validate(Currency, %{currency: \"EUR\"})\n    {:error, %{currency: \"invalid enum type\"}}\n\nAnd for dump the data:\n\n    iex> Parameter.dump(Currency, %{currency: :EUR})\n    {:ok, %{\"currency\" => \"EUR\"}}\n    iex> Parameter.dump(Currency, %{currency: :BRL})\n    {:error, %{currency: \"invalid enum type\"}}\n    # Using the string version should also return an error since it's expected enum values to be atoms\n    iex> Parameter.dump(Currency, %{currency: \"EUR\"})\n    {:error, %{currency: \"invalid enum type\"}}","ref":"Parameter.Enum.html#module-dump-and-validate"},{"type":"behaviour","title":"Parameter.Parametrizable","doc":"Custom types for fields can be done by implementing the `Parameter.Parametrizable` behaviour.\nThis is useful when the basic types provided by `Parameter.Types` are not enough for loading, validating and dumping data.","ref":"Parameter.Parametrizable.html"},{"type":"behaviour","title":"Examples - Parameter.Parametrizable","doc":"To create a parameterized type, create a module as shown below:\n    defmodule MyApp.CustomType do\n      use Parameter.Parametrizable\n\n      @impl true\n      def load(value) do\n        {:ok, value}\n      end\n\n      @impl true\n      def validate(_value) do\n        :ok\n      end\n\n      @impl true\n      def dump(value) do\n        {:ok, value}\n      end\n    end\n\nThen use the new custom type on a param schema:\n    param MyApp.CustomParam do\n      field :custom_field, MyApp.CustomType, key: \"customField\"\n    end\n\nIn general is not necessary to implement dump function since using the macro `use Parameter.Parametrizable`\nwill already use the validate function to dump the value as a default implementation.","ref":"Parameter.Parametrizable.html#module-examples"},{"type":"callback","title":"Parameter.Parametrizable.dump/1","doc":"","ref":"Parameter.Parametrizable.html#c:dump/1"},{"type":"callback","title":"Parameter.Parametrizable.load/1","doc":"","ref":"Parameter.Parametrizable.html#c:load/1"},{"type":"callback","title":"Parameter.Parametrizable.validate/1","doc":"","ref":"Parameter.Parametrizable.html#c:validate/1"},{"type":"module","title":"Parameter.Types.AnyType","doc":"Any parameter type. It will accept any input value without validations.\nThe same value loaded will be the same dumped.","ref":"Parameter.Types.AnyType.html"},{"type":"function","title":"Parameter.Types.AnyType.dump/1","doc":"`AnyType` type will just return the same type value that is passed to dump function","ref":"Parameter.Types.AnyType.html#dump/1"},{"type":"function","title":"Examples - Parameter.Types.AnyType.dump/1","doc":"iex> Parameter.Types.AnyType.load(:any_atom)\n    {:ok, :any_atom}\n\n    iex> Parameter.Types.AnyType.load(\"some string\")\n    {:ok, \"some string\"}\n\n    iex> Parameter.Types.AnyType.load(nil)\n    {:ok, nil}","ref":"Parameter.Types.AnyType.html#dump/1-examples"},{"type":"function","title":"Parameter.Types.AnyType.load/1","doc":"`Any` type will just return the same type value that is passed to load function","ref":"Parameter.Types.AnyType.html#load/1"},{"type":"function","title":"Examples - Parameter.Types.AnyType.load/1","doc":"iex> Parameter.Types.AnyType.load(:any_atom)\n    {:ok, :any_atom}\n\n    iex> Parameter.Types.AnyType.load(\"some string\")\n    {:ok, \"some string\"}\n\n    iex> Parameter.Types.AnyType.load(nil)\n    {:ok, nil}","ref":"Parameter.Types.AnyType.html#load/1-examples"},{"type":"function","title":"Parameter.Types.AnyType.validate/1","doc":"Always return `:ok`, any value is valid","ref":"Parameter.Types.AnyType.html#validate/1"},{"type":"module","title":"Parameter.Types.Array","doc":"Array parameter type","ref":"Parameter.Types.Array.html"},{"type":"module","title":"Parameter.Types.Atom","doc":"Atom parameter type","ref":"Parameter.Types.Atom.html"},{"type":"function","title":"Parameter.Types.Atom.load/1","doc":"loads atom type","ref":"Parameter.Types.Atom.html#load/1"},{"type":"function","title":"Examples - Parameter.Types.Atom.load/1","doc":"iex> Parameter.Types.Atom.load(:atom)\n    {:ok, :atom}\n\n    iex> Parameter.Types.Atom.load(\"atom\")\n    {:ok, :atom}\n\n    iex> Parameter.Types.Atom.load(nil)\n    {:error, \"invalid atom type\"}\n\n    iex> Parameter.Types.Atom.load(123)\n    {:error, \"invalid atom type\"}","ref":"Parameter.Types.Atom.html#load/1-examples"},{"type":"function","title":"Parameter.Types.Atom.validate/1","doc":"validate atom type","ref":"Parameter.Types.Atom.html#validate/1"},{"type":"function","title":"Examples - Parameter.Types.Atom.validate/1","doc":"iex> Parameter.Types.Atom.validate(:atom)\n    :ok\n\n    iex> Parameter.Types.Atom.validate(\"atom\")\n    {:error, \"invalid atom type\"}\n\n    iex> Parameter.Types.Atom.validate(nil)\n    {:error, \"invalid atom type\"}\n\n    iex> Parameter.Types.Atom.validate(123)\n    {:error, \"invalid atom type\"}","ref":"Parameter.Types.Atom.html#validate/1-examples"},{"type":"module","title":"Parameter.Types.Boolean","doc":"Boolean parameter type","ref":"Parameter.Types.Boolean.html"},{"type":"function","title":"Parameter.Types.Boolean.load/1","doc":"loads boolean type","ref":"Parameter.Types.Boolean.html#load/1"},{"type":"function","title":"Examples - Parameter.Types.Boolean.load/1","doc":"iex> Parameter.Types.Boolean.load(true)\n    {:ok, true}\n\n    iex> Parameter.Types.Boolean.load(\"true\")\n    {:ok, true}\n\n    iex> Parameter.Types.Boolean.load(\"True\")\n    {:ok, true}\n\n    iex> Parameter.Types.Boolean.load(\"false\")\n    {:ok, false}\n\n    iex> Parameter.Types.Boolean.load(1)\n    {:ok, true}\n\n    iex> Parameter.Types.Boolean.load(0)\n    {:ok, false}\n\n    iex> Parameter.Types.Boolean.load(\"not boolean\")\n    {:error, \"invalid boolean type\"}\n\n    iex> Parameter.Types.Boolean.load(:not_boolean)\n    {:error, \"invalid boolean type\"}","ref":"Parameter.Types.Boolean.html#load/1-examples"},{"type":"function","title":"Parameter.Types.Boolean.validate/1","doc":"validate boolean type","ref":"Parameter.Types.Boolean.html#validate/1"},{"type":"function","title":"Examples - Parameter.Types.Boolean.validate/1","doc":"iex> Parameter.Types.Boolean.validate(true)\n    :ok\n\n    iex> Parameter.Types.Boolean.validate(false)\n    :ok\n\n    iex> Parameter.Types.Boolean.validate(\"true\")\n    {:error, \"invalid boolean type\"}\n\n    iex> Parameter.Types.Boolean.validate(nil)\n    {:error, \"invalid boolean type\"}\n\n    iex> Parameter.Types.Boolean.validate(123)\n    {:error, \"invalid boolean type\"}","ref":"Parameter.Types.Boolean.html#validate/1-examples"},{"type":"module","title":"Parameter.Types.Date","doc":"Date parameter type","ref":"Parameter.Types.Date.html"},{"type":"function","title":"Parameter.Types.Date.load/1","doc":"loads Date type","ref":"Parameter.Types.Date.html#load/1"},{"type":"function","title":"Examples - Parameter.Types.Date.load/1","doc":"iex> Parameter.Types.Date.load(%Date{year: 1990, month: 5, day: 1})\n    {:ok, ~D[1990-05-01]}\n\n    iex> Parameter.Types.Date.load({2020, 10, 5})\n    {:ok, ~D[2020-10-05]}\n\n    iex> Parameter.Types.Date.load(\"2015-01-23\")\n    {:ok, ~D[2015-01-23]}\n\n    iex> Parameter.Types.Date.load(\"2015-25-23\")\n    {:error, \"invalid date type\"}\n\n    iex> Parameter.Types.Date.load(:not_valid_type)\n    {:error, \"invalid date type\"}","ref":"Parameter.Types.Date.html#load/1-examples"},{"type":"function","title":"Parameter.Types.Date.validate/1","doc":"validate date type","ref":"Parameter.Types.Date.html#validate/1"},{"type":"function","title":"Examples - Parameter.Types.Date.validate/1","doc":"iex> Parameter.Types.Date.validate(%Date{year: 1990, month: 5, day: 1})\n    :ok\n\n    iex> Parameter.Types.Date.validate(~D[1990-05-01])\n    :ok\n\n    iex> Parameter.Types.Date.validate(\"2015-01-23\")\n    {:error, \"invalid date type\"}","ref":"Parameter.Types.Date.html#validate/1-examples"},{"type":"module","title":"Parameter.Types.DateTime","doc":"DateTime parameter type","ref":"Parameter.Types.DateTime.html"},{"type":"function","title":"Parameter.Types.DateTime.load/1","doc":"loads DateTime type","ref":"Parameter.Types.DateTime.html#load/1"},{"type":"function","title":"Examples - Parameter.Types.DateTime.load/1","doc":"iex> Parameter.Types.DateTime.load(~U[2018-11-15 10:00:00Z])\n    {:ok, ~U[2018-11-15 10:00:00Z]}\n\n    iex> Parameter.Types.DateTime.load(\"2015-01-23T23:50:07Z\")\n    {:ok, ~U[2015-01-23 23:50:07Z]}\n\n    iex> Parameter.Types.DateTime.load(\"2015-25-23\")\n    {:error, \"invalid datetime type\"}","ref":"Parameter.Types.DateTime.html#load/1-examples"},{"type":"function","title":"Parameter.Types.DateTime.validate/1","doc":"validate date type","ref":"Parameter.Types.DateTime.html#validate/1"},{"type":"function","title":"Examples - Parameter.Types.DateTime.validate/1","doc":"iex> Parameter.Types.DateTime.validate(~U[2018-11-15 10:00:00Z])\n    :ok\n\n    iex> Parameter.Types.DateTime.validate(~D[1990-05-01])\n    {:error, \"invalid datetime type\"}\n\n    iex> Parameter.Types.DateTime.validate(\"2015-01-23T23:50:07Z\")\n    {:error, \"invalid datetime type\"}","ref":"Parameter.Types.DateTime.html#validate/1-examples"},{"type":"module","title":"Parameter.Types.Decimal","doc":"Decimal parameter type.\nInclude the Decimal library on your application to use this type:\n    def deps do\n      [\n        {:parameter, \"~> ...\"},\n        {:decimal, \"~> 2.0\"}\n      ]\n    end","ref":"Parameter.Types.Decimal.html"},{"type":"module","title":"Parameter.Types.Float","doc":"Float parameter type","ref":"Parameter.Types.Float.html"},{"type":"function","title":"Parameter.Types.Float.load/1","doc":"loads float type","ref":"Parameter.Types.Float.html#load/1"},{"type":"function","title":"Examples - Parameter.Types.Float.load/1","doc":"iex> Parameter.Types.Float.load(1.5)\n    {:ok, 1.5}\n\n    iex> Parameter.Types.Float.load(\"2.5\")\n    {:ok, 2.5}\n\n    iex> Parameter.Types.Float.load(1)\n    {:ok, 1.0}\n\n    iex> Parameter.Types.Float.load(\"not float\")\n    {:error, \"invalid float type\"}\n\n    iex> Parameter.Types.Float.load(:atom)\n    {:error, \"invalid float type\"}","ref":"Parameter.Types.Float.html#load/1-examples"},{"type":"module","title":"Parameter.Types.Integer","doc":"Integer parameter type","ref":"Parameter.Types.Integer.html"},{"type":"module","title":"Parameter.Types.Map","doc":"Map parameter type","ref":"Parameter.Types.Map.html"},{"type":"module","title":"Parameter.Types.NaiveDateTime","doc":"NaiveDateTime parameter type","ref":"Parameter.Types.NaiveDateTime.html"},{"type":"function","title":"Parameter.Types.NaiveDateTime.load/1","doc":"loads NaiveDateTime type","ref":"Parameter.Types.NaiveDateTime.html#load/1"},{"type":"function","title":"Examples - Parameter.Types.NaiveDateTime.load/1","doc":"iex> Parameter.Types.NaiveDateTime.load(~N[2000-01-01 23:00:07])\n    {:ok, ~N[2000-01-01 23:00:07]}\n\n    iex> Parameter.Types.NaiveDateTime.load(\"2000-01-01 22:00:07\")\n    {:ok, ~N[2000-01-01 22:00:07]}\n\n    iex> Parameter.Types.NaiveDateTime.load({{2021, 05, 11}, {22, 30, 10}})\n    {:ok, ~N[2021-05-11 22:30:10]}\n\n    iex> Parameter.Types.NaiveDateTime.load({{2021, 25, 11}, {22, 30, 10}})\n    {:error, \"invalid naive_datetime type\"}\n\n    iex> Parameter.Types.NaiveDateTime.load(\"2015-25-23\")\n    {:error, \"invalid naive_datetime type\"}","ref":"Parameter.Types.NaiveDateTime.html#load/1-examples"},{"type":"module","title":"Parameter.Types.String","doc":"String parameter type","ref":"Parameter.Types.String.html"},{"type":"module","title":"Parameter.Types.Time","doc":"Time parameter type","ref":"Parameter.Types.Time.html"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.14.1 (2024-09-09) - Changelog","doc":"#","ref":"changelog.html#v0-14-1-2024-09-09"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Parameter.Enum] Added examples for `Parameter.validate/3` and `Parameter.dump/3` for enum values.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.14.0 (2024-08-21) - Changelog","doc":"#","ref":"changelog.html#v0-14-0-2024-08-21"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- [Parameter] Fix `nil` default values not being shown on load/dump output.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.13.x (2023-06-19) - Changelog","doc":"#","ref":"changelog.html#v0-13-x-2023-06-19"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"- [Parameter] `on_load/2` or `on_dump/2` now always run when parsing the schema. Previously, both functions were only triggered when the field was present on the params.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"v0.12.x (2023-03-06) - Changelog","doc":"#","ref":"changelog.html#v0-12-x-2023-03-06"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Parameter.Schema.Compiler] Unify schema compiler for macro and functional API.\n- [Parameter.Types.Any] Renamed module to `Parameter.Types.AnyType` to fix elixir warnings\n- [Parameter] Fix `load`, `dump` and `validate` functions to correctly parse Enum values in nested schema.\n- [Parameter.Schema] default options for nested schema is now available.\n- [Parameter.Schema] Add a description on what `use Parameter.Schema` does.\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Deprecations - Changelog","doc":"- [Parameter.Types.List] removed type in favor of `Array` type","ref":"changelog.html#deprecations"},{"type":"extras","title":"v0.11.x (2023-01-18) - Changelog","doc":"#","ref":"changelog.html#v0-11-x-2023-01-18"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Parameter] Extra option `ignore_empty` for `load/2` and `dump/2` functions.\n- [Parameter.Schema] `has_one` and `has_many` nested types as `map` and `array` composite types.\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Deprecations - Changelog","doc":"- [Parameter.Enum] removed enum value macro with the `as` key.","ref":"changelog.html#deprecations"},{"type":"extras","title":"v0.10.x (2023-01-07) - Changelog","doc":"#","ref":"changelog.html#v0-10-x-2023-01-07"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Parameter] Extra option `ignore_nil` for `load/2` and `dump/2` functions.\n- [Parameter.Types] New `array` type.\n- [Parameter.Schema] Support for nested type on `map` and `array`.\n- Improved code test coverage to 100%.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.9.x (2023-01-04) - Changelog","doc":"#","ref":"changelog.html#v0-9-x-2023-01-04"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Parameter.Field] Support for `on_load/2` and `on_dump/2` functions in field definition.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.8.x (2022-12-10) - Changelog","doc":"#","ref":"changelog.html#v0-8-x-2022-12-10"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Parameter.Schema] Supports `compile/1` function for compiling runtime schemas.\n- [Parameter] `load/3`, `validate/3` and `dump/3` now support evaluating parameters using runtime schemas.\n- [Parameter.Validators] Improved `length/2` validator to support `min` and/or `max` attributes. Before it was only accepting both.\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- [Parameter] Fix a bug where `load/3` and `validate/3` was evaluating the `validator` option wrongly.\n- [Parameter.Field] Validator typespec.\n- [Parameter.Enum] Fix evaluation of enum values inside `Enum` macro","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.7.x (2022-11-07) - Changelog","doc":"#","ref":"changelog.html#v0-7-x-2022-11-07"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Parameter] New `validate/3` function\n- [Parameter] Supports for `load/3` parsing maps with atom keys\n- [Parameter.Schema] Supports for `fields_required` module attribute on schema\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- [Parameter] `dump/3` function to load the value to be dumped\n- [Parameter] consider basic types when loading, dumping or validating a schema\n- [Parameter.Field] remove compile time verification for custom types","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.6.x (2022-11-06) - Changelog","doc":"#","ref":"changelog.html#v0-6-x-2022-11-06"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Parameter] API changes to support new [parameter_ecto](https://github.com/phcurado/parameter_ecto) library\n- [Parameter] Support for `many` flag on `load/3` and `dump/3` options\n- [Parameter] Errors when parsing list return as map with `%{index => reason}` now instead of `{:#{index}, reason}` to avoid atom creation\n- [Parameter.Field] Support for `load_default` and `dump_default` options\n- [Parameter.Enum] Deprecated `as` in favour of `key`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- [Parameter.Field] Return `default` value when calling `Parameter.dump/3` with empty value.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.5.x (2022-10-26) - Changelog","doc":"#","ref":"changelog.html#v0-5-x-2022-10-26"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Parameter] Support for `exclude` option for `load/3` and `dump/3`.\n- [Parameter.Types] Support for `any` type\n- [Parameter.Schema] Support for virtual fields","ref":"changelog.html#enhancements"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}